# Copyright (c) 2026 Shon Ness
#
# This source code is licensed under the MIT License found in the
# LICENSE file in the root directory of this source tree.

__version__ = "0.0.2"
__author__ = "nessshon"
__url__ = "https://github.com/nessshon/ton-connect"
